<?php
/**
 * ILS driver test
 *
 * PHP version 7
 *
 * Copyright (C) Leipzig University Library 2020.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * @category VuFind
 * @package  Tests
 * @author   Robert Lange <lange@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org Main Page
 */
namespace VuFindTest\ILS\Driver;

use finc\ILS\Driver\PAIA as fincPAIA;
use InvalidArgumentException;
use Zend\Http\Client\Adapter\Test as TestAdapter;
use Zend\Http\Response as HttpResponse;

/**
 * ILS driver test
 *
 * @category VuFind
 * @package  FincTest
 * @author   Robert Lange <lange@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org Main Page
 */
class PAIAFincTest extends \VuFindTest\ILS\Driver\PAIATest
{
    public static $pickupCondition = 'http://purl.org/ontology/paia#PickupCondition';

    public static $validConfigWithPickUpOption = [
        'DAIA' =>
            [
                'baseUrl'           => 'http://daia.gbv.de/',
            ],
        'PAIA' =>
            [
                'baseUrl'           => 'http://paia.gbv.de/',
                'paiaConditions'    =>
                    [
                        'http://purl.org/ontology/paia#PickupCondition'
                    ]
            ]
    ];

    public static $validConfigWithOptionForOtherInstance = [
        'DAIA' =>
            [
                'baseUrl'           => 'http://daia.gbv.de/',
            ],
        'PAIA' =>
            [
                'baseUrl'           => 'http://paia.gbv.de/',
                'paiaConditions'    =>
                    [
                        'http://purl.org/ontology/paia#PickupCondition' =>  'http://data.ub.uni-leipzig.de/resource/DE-15/department/wrongPickup'
                    ]
            ]
    ];

    /**
     * Test
     *
     * @return void
     */
    public function testMapOptionsPickupBranchTrue()
    {
        list($method, $paia) = $this->prepareByReflection();

        $pickUpId       = 'http://data.ub.uni-leipzig.de/resource/DE-Zi4/department/zwGR';
        $pickUpAbout    = 'Zweigbibliothek Görlitz';
        $input  = [
            'condition' => [
                self::$pickupCondition => [
                    'option' => [
                        0 => [
                            'id'    => $pickUpId,
                            'about' => $pickUpAbout
                        ]
                    ]
                ]
            ]
        ];

        $conditions = [
            self::$pickupCondition
        ];

        $output = [];
        $method->invokeArgs($paia, [$input, &$output, $conditions]);

        $this->assertEquals($pickUpId, $output['options'][self::$pickupCondition][0]['id'],
            "Wrong pickup branch id!"
        );

        $this->assertEquals($pickUpAbout, $output['options'][self::$pickupCondition][0]['about'],
            "Wrong pickup branch label!"
        );
    }

    /**
     * Test
     *
     * @return void
     */
    public function testMapOptionsNewTrue()
    {
        list($method, $paia) = $this->prepareByReflection();

        $pickUpId       = 'http://data.ub.uni-leipzig.de/resource/DE-Zi4/department/zwGR';
        $pickUpAbout    = 'Zweigbibliothek Görlitz';
        $newOptionId    = 'http://data.ub.uni-leipzig.de/resource/DE-Zi4/newOption';
        $newOption      = 'a new option within new condition';
        $input  = [
            'condition' => [
                self::$pickupCondition => [
                    'option' => [
                        0 => [
                            'id'    => $pickUpId,
                            'about' => $pickUpAbout
                        ]
                    ]
                ],
                'http://purl.org/ontology/paia#newCondition' => [
                    'option' => [
                        0 => [
                            'id' => $newOptionId,
                            'about' => $newOption
                        ]
                    ]
                ]
            ]
        ];

        $conditions = [
            self::$pickupCondition,
            'http://purl.org/ontology/paia#newCondition'
        ];

        $output = [];
        $method->invokeArgs($paia, [$input, &$output, $conditions]);
        $this->assertEquals($pickUpId, $output['options'][self::$pickupCondition][0]['id'],
            "Wrong pickup branch id!"
        );
        $this->assertEquals($pickUpAbout, $output['options'][self::$pickupCondition][0]['about'],
            "Wrong pickup branch label!"
        );
        $this->assertEquals($newOptionId, $output['options']['http://purl.org/ontology/paia#newCondition'][0]['id'],
            "New option id missing!"
        );
        $this->assertEquals($newOption, $output['options']['http://purl.org/ontology/paia#newCondition'][0]['about'],
            "New option label missing!"
        );
    }

    /**
     * Test
     *
     * @return void
     */
    public function testMapOptionsPickupBranchFalseNoConditions()
    {
        list($method, $paia) = $this->prepareByReflection();

        $pickUpBranch = 'Zweigbibliothek Görlitz';
        $input  = [
            'condition' => [
                self::$pickupCondition => [
                    'option' => [
                        0 => [
                            'id' => 'http://data.ub.uni-leipzig.de/resource/DE-Zi4/department/zwGR',
                            'about' => $pickUpBranch
                        ]
                    ]
                ]
            ]
        ];

        $output = [];
        $method->invokeArgs($paia, [$input, &$output, []]);
        $this->assertFalse(isset($output["options"]),
            "Unknown option was mapped"
        );
    }

    /**
     * Test
     *
     * @return void
     */
    public function testMapOptionsPickupBranchTrueUnknownOptionId()
    {
        /* validate new options otherwise, e.g. by checking $presentationPickups later */
        list($method, $paia) = $this->prepareByReflection();

        $pickUpBranch = 'Zweigbibliothek Görlitz';
        $input  = [
            'condition' => [
                self::$pickupCondition => [
                    'option' => [
                        0 => [
                            'id' => 'http://data.ub.uni-leipzig.de/resource/DE-Zi4/department/fake',
                            'about' => $pickUpBranch
                        ]
                    ]
                ]
            ]
        ];

        $conditions = [
            self::$pickupCondition
        ];

        $output = [];
        $method->invokeArgs($paia, [$input, &$output, $conditions]);
        $this->assertTrue(isset($output["options"]),
            "Unknown option was not mapped"
        );
    }

    /**
     * Test
     *
     * @return void
     */
    public function testGetMyHoldsWithoutOptionConfigurationDefined()
    {
        $conn = $this->createConnector('fincWithOnePickupBranchOption.json');
        /* default config */
        $conn->setConfig($this->validConfig);
        $conn->init();
        $result = $conn->getMyHolds($this->patron);

        $filtered = array_filter(
            $result,
            function ($resultItem) {
                if (isset($resultItem['options'])) {
                    return true;
                }
            }
        );

        /* no option or pickup branch */
        $this->assertTrue(count($filtered) == 0,
            "Two many items with options!"
        );
    }

    /**
     * Test
     *
     * @return void
     */
    public function testGetMyHoldsWithoutPaiaResponseOptions()
    {
        $conn = $this->createConnector('fincWithoutOptions.json');
        $conn->setConfig(self::$validConfigWithPickUpOption);
        $conn->init();
        $result = $conn->getMyHolds($this->patron);

        $filtered = array_filter(
            $result,
            function ($resultItem) {
                if (isset($resultItem['options'])) {
                    return true;
                }
            }
        );

        /* no option nor pickup branch */
        $this->assertTrue(count($filtered) == 0,
            "Two many items with options!"
        );
    }

    /**
     * Test
     *
     * @return void
     */
    public function testGetMyHoldsAllowUnknownOptions()
    {
        $conn = $this->createConnector('fincWithUnknownOption.json');
        /* default config */
        $conn->setConfig(self::$validConfigWithPickUpOption);
        $conn->init();
        $result = $conn->getMyHolds($this->patron);

        $filtered = array_filter(
            $result,
            function ($resultItem) {
                if (isset($resultItem['options'])) {
                    return true;
                }
            }
        );

        /* no option */
        $this->assertTrue(count($filtered) > 0,
            "Two many items with options!"
        );
    }

    /**
     * Test
     *
     * @return void
     */
    public function testGetMyHoldsWrongInstance()
    {
        $conn = $this->createConnector('fincWithOnePickupBranchOption.json');
        /* default config */
        $conn->setConfig(self::$validConfigWithOptionForOtherInstance);
        $conn->init();
        $result = $conn->getMyHolds($this->patron);

        $filtered = array_filter(
            $result,
            function ($resultItem) {
                if (isset($resultItem['options'])) {
                    return true;
                }
            }
        );

        /* no option */
        $this->assertTrue(count($filtered) == 0,
            "Two many items with options!"
        );
    }

    /**
     * Test
     *
     * @return void
     */
    public function testGetMyHoldsWithPickUpBranch()
    {
        $conn = $this->createConnector('fincWithOnePickupBranchOption.json');
        $conn->setConfig(self::$validConfigWithPickUpOption);
        $conn->init();
        $result = $conn->getMyHolds($this->patron);

        $filtered = array_filter(
            $result,
            function ($resultItem) {
                if (isset($resultItem['options'])) {
                    return true;
                }
            }
        );

        /* check pickupbranch */
        $this->assertTrue(count($filtered[0]["options"][self::$pickupCondition]) > 0,
            "No pickup branch mapped!");

        $this->assertEquals("Zweigbibliothek Görlitz", $filtered[0]["options"][self::$pickupCondition][0]["about"],
            "Wrong pickup branch!"
        );

        /* but keep storage id (location) */
        $this->assertEquals("Hochschulbibliothek Zittau", $filtered[0]["location"],
            "Wrong pickup branch!"
        );
    }

    /**
     * Test
     *
     * @return void
     */
    public function testGetMyHoldsWithAnotherPickUpBranch()
    {
        $conn = $this->createConnector('fincWithAnotherPickupBranchOption.json');
        $conn->setConfig(self::$validConfigWithPickUpOption);
        $conn->init();
        $result = $conn->getMyHolds($this->patron);

        $filtered = array_filter(
            $result,
            function ($resultItem) {
                if (isset($resultItem['options'])) {
                    return true;
                }
            }
        );

        /* check pickupbranch */
        $this->assertTrue(count($filtered[0]["options"][self::$pickupCondition]) > 0,
            "No pickup branch mapped!");

        $this->assertEquals("http://data.ub.uni-leipzig.de/resource/DE-Zi4/department/zwZI", $filtered[0]["options"][self::$pickupCondition][0]["id"],
            "Wrong pickup branch id!"
        );

        $this->assertEquals("Zweigbibliothek Zittau", $filtered[0]["options"][self::$pickupCondition][0]["about"],
            "Wrong pickup branch label!"
        );

        $this->assertEquals("2,50 EUR", $filtered[0]["options"][self::$pickupCondition][0]["amount"],
            "Wrong pickup branch amount!"
        );

        /* but keep storage id (location) */
        $this->assertEquals("Hochschulbibliothek Görlitz", $filtered[0]["location"],
            "Wrong pickup branch!"
        );
    }

    /**
     * Test Multiple Options => use first option when displaying items
     *
     * @return void
     */
    public function testGetMyHoldsWithMultipleOptionsPickUpBranch()
    {
        $conn = $this->createConnector('fincWithMultiplePickupBranchOptions.json');
        $conn->setConfig(self::$validConfigWithPickUpOption);
        $conn->init();
        $result = $conn->getMyHolds($this->patron);

        $filtered = array_filter(
            $result,
            function ($resultItem) {
                if (isset($resultItem['options'])) {
                    return true;
                }
            }
        );

        /* check pickupbranch */
        $this->assertTrue(count($filtered) == 1,
            "No pickup branch mapped!");

        $this->assertEquals("http://data.ub.uni-leipzig.de/resource/DE-Zi4/department/zwZI", $filtered[0]["options"][self::$pickupCondition][0]["id"],
            "Wrong pickup branch id!"
        );

        $this->assertEquals("Zweigbibliothek Zittau", $filtered[0]["options"][self::$pickupCondition][0]["about"],
            "Wrong pickup branch label!"
        );

        $this->assertEquals("http://data.ub.uni-leipzig.de/resource/DE-Zi4/department/zwGR", $filtered[0]["options"][self::$pickupCondition][1]["id"],
            "Wrong pickup branch id!"
        );

        $this->assertEquals("Zweigbibliothek Görlitz", $filtered[0]["options"][self::$pickupCondition][1]["about"],
            "Wrong pickup branch label!"
        );

        /* but keep storage id (location) */
        $this->assertEquals("Hochschulbibliothek Görlitz", $filtered[0]["location"],
            "Wrong pickup branch!"
        );
    }

    /**
     * Test
     *
     * @return void
     */
    public function testgetOptionsReturnsIDTrue()
    {
        $conn = $this->createConnector('fincWithMultiplePickupBranchOptions.json');
        $conn->setConfig(self::$validConfigWithPickUpOption);
        $conn->init();
        $options['options'][self::$pickupCondition][0]['id'] = 'TESTID';

        $this->assertEquals(
            ['TESTID'],
            $conn->getOptions(self::$pickupCondition, $options, true),
            "Wrong option value"
        );

        $validOptions[0][0] = 'TESTID';
        $this->assertEquals(
            ['TESTID'],
            $conn->getOptions(self::$pickupCondition, $options, true, $validOptions),
            "Wrong option value"
        );
    }

    /**
     * Test
     *
     * @return void
     */
    public function testgetOptionsNotOnlyFirstButAll()
    {
        $conn = $this->createConnector('fincWithMultiplePickupBranchOptions.json');
        $conn->setConfig(self::$validConfigWithPickUpOption);
        $conn->init();
        $options['options'][self::$pickupCondition][0]['id'] = 'TESTID1';
        $options['options'][self::$pickupCondition][1]['id'] = 'TESTID2';

        $this->assertEquals(
            ['TESTID1', 'TESTID2'],
            $conn->getOptions(self::$pickupCondition, $options, false),
            "Wrong option value"
        );

        $validOptions[0][0] = 'TESTID1';
        $validOptions[0][1] = 'TESTID2';
        $this->assertEquals(
            ['TESTID1', 'TESTID2'],
            $conn->getOptions(self::$pickupCondition, $options, false, $validOptions),
            "Wrong option value"
        );

        /* first option only label set */
        $options['options'][self::$pickupCondition][0]['id']    = null;
        $options['options'][self::$pickupCondition][0]['about'] = 'TESTLABEL1';
        $options['options'][self::$pickupCondition][1]['id']    = 'TESTID2';

        $this->assertEquals(
            ['TESTLABEL1', 'TESTID2'],
            $conn->getOptions(self::$pickupCondition, $options, false, null),
            "Wrong option value"
        );
    }

    /**
     * Test
     *
     * @return void
     */
    public function testgetOptionsReturnsLabelTrue()
    {
        $conn = $this->createConnector('fincWithMultiplePickupBranchOptions.json');
        $conn->setConfig(self::$validConfigWithPickUpOption);
        $conn->init();
        $options['options'][self::$pickupCondition][0]['id'] = null;
        $options['options'][self::$pickupCondition][0]['amount'] = '2,50 Euro';
        $options['options'][self::$pickupCondition][0]['about'] = 'use label when id missing';

        $this->assertEquals(
            ['use label when id missing'],
            $conn->getOptions(self::$pickupCondition, $options, true),
            "Wrong option value!"
        );

        /* also return label instead of id when no valid options are given */
        $options['options'][self::$pickupCondition][0]['id'] = 'http://data.ub.uni-leipzig.de/resource/DE-Zi4/department/zwGR';
        $options['options'][self::$pickupCondition][0]['about'] = 'Take label, if parameter valid Options missing';
        $this->assertEquals(
            ['Take label, if parameter valid Options missing'],
            $conn->getOptions(self::$pickupCondition, $options, true),
            "Wrong option value!"
        );
    }

    /**
     * Test
     *
     * @return void
     */
    public function testgetOptionsReturnsEmptyArray()
    {
        $conn = $this->createConnector('fincWithMultiplePickupBranchOptions.json');
        $conn->setConfig(self::$validConfigWithPickUpOption);
        $conn->init();

        $options['options'][self::$pickupCondition][0]['id'] = 'http://data.ub.uni-leipzig.de/resource/DE-Zi4/department/zwGR';
        $options['options'][self::$pickupCondition][0]['about'] = 'Zweigbibliothek Görlitz';

        /* invalid condition */
        $this->assertEquals(
            [],
            $conn->getOptions('fakeCondition', $options, true),
            "Wrong option value!"
        );

        /* no options in result array */
        $this->assertEquals(
            [],
            $conn->getOptions('fakeCondition', [], true),
            "Wrong option value!"
        );

        /* invalid option */
        $validOptions[0][0] = 'xxx';
        $this->assertEquals(
            [],
            $conn->getOptions(self::$pickupCondition, $options, true, $validOptions),
            "Wrong option value!"
        );

        /* no option values */
        $options['options'][self::$pickupCondition][0]['id'] = null;
        $options['options'][self::$pickupCondition][0]['about'] = null;
        $this->assertEquals(
            [""],
            $conn->getOptions(self::$pickupCondition, $options, true),
            "Wrong option value!"
        );
    }

    /**
     * Test
     *
     * @return void
     */
    public function testCallMapOptionsTrue() : void
    {
        $paia = $this->getMockBuilder(PAIAMock::class)
            ->enableProxyingToOriginalMethods()
            ->setMethods(['mapOptions'])
            ->setConstructorArgs([new \VuFind\Date\Converter(), new \Zend\Session\SessionManager()])
            ->getMock();

        $paia->prepare("fincWithOnePickupBranchOption.json");

        $paia->expects($this->atLeastOnce())
            ->method('mapOptions');

        /* only used by holdings so far */
        $paia->getMyHolds($this->patron);
    }

    /**
     * Test
     *
     * @return void
     */
    public function testCallMapOptionsFalse() : void
    {
        $paia = $this->getMockBuilder(PAIAMock::class)
            ->enableProxyingToOriginalMethods()
            ->setMethods(['mapOptions'])
            ->setConstructorArgs([new \VuFind\Date\Converter(), new \Zend\Session\SessionManager()])
            ->getMock();

        $paia->prepare("fincWithOnePickupBranchOption.json");

        $paia->expects($this->never())
            ->method('mapOptions');

        $paia->getMyILLRequests($this->patron);
        $paia->getMyFines($this->patron);
        $paia->getMyStorageRetrievalRequests($this->patron);
        $paia->getMyTransactions($this->patron);
    }

    /**
     * Create connector with fixture file.
     *
     * @param string $fixture Fixture file
     *
     * @return Connector
     *
     * @throws InvalidArgumentException Fixture file does not exist
     */
    protected function createConnector($fixture = null)
    {
        if (empty($fixture) || strpos($fixture, 'finc') === false) {
            return parent::createConnector($fixture);
        }

        $adapter = new TestAdapter();
        if ($fixture) {
            $file = realpath(
                __DIR__ .
                '/../../../../../fixtures/paia/response/' . $fixture
            );
            if (!is_string($file) || !file_exists($file) || !is_readable($file)) {
                throw new InvalidArgumentException(
                    sprintf('Unable to load fixture file: %s ', $file)
                );
            }
            $response = file_get_contents($file);
            $responseObj = HttpResponse::fromString($response);
            $adapter->setResponse($responseObj);
        }
        $service = new \VuFindHttp\HttpService();
        $service->setDefaultAdapter($adapter);
        $conn = new PAIAMock(
            new \VuFind\Date\Converter(),
            new \Zend\Session\SessionManager()
        );
        $conn->setHttpService($service);
        return $conn;
    }

    /**
     * @return array
     * @throws \ReflectionException
     */
    protected function prepareByReflection()
    {
        /* use reflection due to protected access level */
        $class = new \ReflectionClass('finc\ILS\Driver\PAIA');
        $method = $class->getMethod("mapOptions");
        $method->setAccessible(true);

        $paia = new fincPAIA(
            new \VuFind\Date\Converter(),
            new \Zend\Session\SessionManager()
        );

        return [$method, $paia];
    }
}

/* helper class to make protected finc methods public and reuse Vufind mocking approach */
class PAIAMock extends fincPAIA
{
    public function prepare($fixture)
    {
        $adapter = new TestAdapter();
        if ($fixture) {
            $file = realpath(
                __DIR__ .
                '/../../../../../fixtures/paia/response/' . $fixture
            );
            if (!is_string($file) || !file_exists($file) || !is_readable($file)) {
                throw new InvalidArgumentException(
                    sprintf('Unable to load fixture file: %s ', $file)
                );
            }
            $response = file_get_contents($file);
            $responseObj = HttpResponse::fromString($response);
            $adapter->setResponse($responseObj);
        }
        $service = new \VuFindHttp\HttpService();
        $service->setDefaultAdapter($adapter);
        $this->setHttpService($service);
        $this->setConfig(PAIAFincTest::$validConfigWithPickUpOption);
        parent::init();
    }

    public function parsePAIAJson($file)
    {
        return $this->paiaParseJsonAsArray($file);
    }

    public function myHoldsMapping($items)
    {
        return parent::myHoldsMapping($items);
    }

    public function paiaGetItems($patron, $filter = [])
    {
        return parent::paiaGetItems($patron, $filter);
    }

    public function paiaCheckScope($scope)
    {
        return true;
    }
}
